<?php

/**
 * Limit recipients
 *
 * Plugin blocks sending messages to more than allowed recipients
 *
 * @version 1.1 - 09.09.2010
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/

/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/

class limit_recipients extends rcube_plugin
{

  function init() {

    $this->_load_config("limit_recipients");
    $rcmail = rcmail::get_instance();
    $this->add_hook('message_outgoing_headers', array($this, 'check_recipients'));

  }

  function _load_config($plugin) {

    $rcmail = rcmail::get_instance();
    $config = "plugins/" . $plugin . "/config/config.inc.php";
    if(file_exists($config))
      include $config;
    else if(file_exists($config . ".dist"))
      include $config . ".dist";
    if(is_array($rcmail_config)){
      $arr = array_merge($rcmail->config->all(),$rcmail_config);
      $rcmail->config->merge($arr);
    }
  }

  function check_recipients($args){

    if(!empty($_POST['_draft']))
      return $args;

    $rcmail = rcmail::get_instance();

    $methods = array_flip((array)get_class_methods($rcmail->imap));
    if(!isset($methods['decode_address_list']))
      return $args;

    $recipients = 0;
    $to = $rcmail->imap->decode_address_list($args['headers']['To']);
    if($to[1]['name'])
      $recipients = $recipients + count($to);
    $cc = $rcmail->imap->decode_address_list($args['headers']['Cc']);
    if($cc[1]['name'])
      $recipients = $recipients + count($cc);
    $bcc  = $rcmail->imap->decode_address_list($args['headers']['Bcc']);
    if($bcc[1]['name'])
      $recipients = $recipients + count($bcc);

    if($recipients > $rcmail->config->get('limit_recipients')){
      if($rcmail->config->get('limit_recipients_log')){
        $this->log("Limit Recipients: " . $recipients . " > " . $rcmail->config->get('limit_recipients') . " --> " . $_SESSION['username'] . " --> IP: " . $this->getVisitorIP());
      }
      $this->add_texts('localization/');
      $rcmail->output->command('display_message',sprintf(rcube_label('limit_recipients_msg','limit_recipients'),$rcmail->config->get('limit_recipients')),'error');
      $rcmail->output->send('iframe');
    }

    return $args;

  }

  function getVisitorIP() {

    //Regular expression pattern for a valid IP address
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/";

    //Retrieve IP address from which the user is viewing the current page
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) {
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR'));
    }
    else {
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR'));
    }

    if (strpos($visitorIP, ',')!==false) {
      $visitorIP = trim(end(explode(',',$visitorIP)));
    }

    return $visitorIP;
  }

  function log($log){

    $rcmail = rcmail::get_instance();

    if($rcmail->config->get('limit_recipients_log')){
      write_log('limit_recipients',$log);
    }
  }
}

?>